#include "camera.h"
#include "render.h"
#include <stdio.h>

WorldRect Camera::validCenters() const
{
    WorldRect b = this->world->worldBorder;

    /* Allow up to just next to the world border */
    return WorldRect(b.xmin + 1, b.xmax - 1, b.ymin + 1, b.ymax - 1);
}

WorldRect Camera::focusedRegion() const
{
    WorldRect b = this->visible;
    return WorldRect(b.xmin / 3, b.xmax / 3, b.ymin / 3, b.ymax / 3);
}

void Camera::moveTo(WorldCoord p)
{
    this->center = this->validCenters().clampPoint(p);
}

void Camera::moveBy(WorldCoord direction)
{
    this->moveTo(this->center + direction);
}

void Camera::moveToFollow(WorldCoord p)
{
    WorldCoord pc = p - this->center;
    WorldRect focused = this->focusedRegion();

    if(focused.contains(pc))
        return;

    /* Determine movement required to put pc into the focused region */
    WorldCoord diff(0, 0);
    if(pc.x < focused.xmin)
        diff.x = pc.x - focused.xmin;
    if(pc.x > focused.xmax)
        diff.x = pc.x - focused.xmax;
    if(pc.y < focused.ymin)
        diff.y = pc.y - focused.ymin;
    if(pc.y > focused.ymax)
        diff.y = pc.y - focused.ymax;

    this->moveBy(diff);
}

namespace Nooncraft {

void renderCamera(int x, int y, Camera *camera, WorldCoord playerPos,
   GlyphCluster playerCluster, WorldCoord cursorPos,
   GlyphCluster cursorCluster)
{
    if(!camera)
        return;

    WorldRect r = camera->visible;
    for(int dy = r.ymin; dy <= r.ymax; dy++)
    for(int dx = r.xmin; dx <= r.xmax; dx++) {
        int wx = camera->center.x + dx;
        int wy = camera->center.y + dy;
        GlyphCluster c;

        if(!camera->world->worldBorder.contains(WorldCoord(wx, wy))) {
            continue;
        }
        else if(WorldCoord(wx, wy) == playerPos) {
            c = playerCluster;
        }
        else if(camera->world->isPointOnWorldBorder(wx, wy)) {
            c = GlyphCluster('{', '}', '{', '}');
        }
        else {
            Block b = camera->world->cellAt(wx, wy);
            BlockInfo *info = Nooncraft::getBlockInfo(b);
            if(info) {
                c = info->cluster;
            }
            else {
                char str[5];
                sprintf(str, "%04X", 0x55);
                c = GlyphCluster(str[0], str[1], str[2], str[3]);
            }
        }

        if(WorldCoord(wx, wy) == cursorPos)
            c = cursorCluster.applyOver(c);
        renderCluster(x+2*dx, y+2*dy, c);
    }
}

} /* namespace Nooncraft */
