// nooncraft.camera: Application-level rendering logic

#pragma once
#include "world.h"

struct Camera
{
    Camera(World const *w): world {w}, center {0,0}, visible {-1,1,-1,1} {}

    /* Underlying world object */
    World const *world;

    /* Coordinates of the center point */
    WorldCoord center;
    /* Local rectangle around center at (0,0). This influences how much of the
       screen the camera will take up when rendered */
    WorldRect visible;

    /* Range of possible values for center */
    WorldRect validCenters() const;
    /* Center rectangle where we allow the player to be */
    WorldRect focusedRegion() const;

    /* Keep focus on a point */
    void moveTo(WorldCoord p);
    /* Move the camera by a fixed vector */
    void moveBy(WorldCoord direction);
    /* Move the camera to ensure `p` is somewhat in frame */
    void moveToFollow(WorldCoord p);
};

namespace Nooncraft {

void renderCamera(int x, int y, Camera *camera, WorldCoord playerPos,
   GlyphCluster playerCluster, WorldCoord cursorPos,
   GlyphCluster cursorCluster);

} /* namespace Nooncraft */
