#include "item.h"

extern "C" {
extern ItemInfo Nooncraft_itemInfo[];
extern int Nooncraft_itemInfoCount;
}

int Item::stackSize() const
{
    if(this->isBlock)
        return 64;
    ItemInfo *info = Nooncraft::getItemInfo(*this);
    return info ? info->stackSize : 64;
}

namespace Nooncraft {

ItemInfo *getItemInfo(Item const &i)
{
    if(i.isBlock)
        return nullptr;
    if((int)i.itemID >= Nooncraft_itemInfoCount)
        return nullptr;
    return &Nooncraft_itemInfo[(int)i.itemID];
}

BlockInfo *getItemBlockInfo(Item const &i)
{
    if(!i.isBlock)
        return nullptr;
    return Nooncraft::getBlockInfo(i.block);
}

GlyphCluster getItemCluster(Item const &i)
{
    if(i.isBlock) {
        BlockInfo *info = Nooncraft::getBlockInfo(i.block);
        return info ? info->cluster : GlyphCluster();
    }
    else {
        if((int)i.itemID < 0)
            return GlyphCluster();
        ItemInfo *info = Nooncraft::getItemInfo(i);
        return info ? info->cluster : GlyphCluster();
    }
}

char const *getItemName(Item const &i)
{
    if(i.isBlock) {
        BlockInfo *info = Nooncraft::getBlockInfo(i.block);
        return info ? info->name : "(BAD BLOCK)";
    }
    else {
        if((int)i.itemID < 0)
            return "(NOTHING)";
        ItemInfo *info = Nooncraft::getItemInfo(i);
        return info ? info->name : "(BAD ITEM)";
    }
}

} /* namespace Nooncraft */

